/* ------------------------------------------------------------------
   Functional (External) Iterator Pattern 
*/

class TreeRange implements IRange {

    /* a list of nodes on the way down */
    ABTIlist ptr; 
    
    TreeRange(ConsBinTreeInt t) { 
        /* insert the current node into the list */
        ptr = new ConsBTlist(t, new EmptyABTIlist());
        
        /* insert all the left children of this node into the list */ 
        ptr = ((ConsBTlist)ptr).insertLeft();        
    }
    // --------------------------------------------------------------


    public void next() {
    
        /* give decent names to the first and rest */
        ConsBinTreeInt t = ((ConsBTlist)ptr).first;
        ABTIlist theRest = ((ConsBTlist)ptr).rest;
        
        /* if the to node had a right child,
           insert the right child into the list
           then insert all of its left children too */
        if (t.right instanceof ConsBinTreeInt){
            ptr = (new ConsBTlist(((ConsBinTreeInt)t.right), 
                                    theRest)).insertLeft();
            }
            
        /* otherwise, just keep the rest of the list */
        else
            ptr = theRest;
    }


    public Object current() {
        return ((ConsBTlist)ptr).first.node; 
    }
    
    public boolean hasMore() {
        return (ptr instanceof ConsBTlist); 
    }
}

/* A list where to keep track of the nodes to visit */        
abstract class ABTIlist{ }

class EmptyABTIlist extends ABTIlist{
    public String toString(){ return("@%@"); }
}

class ConsBTlist extends ABTIlist{
    
    /* insist that only non-empty trees get inserted */
    ConsBinTreeInt first;
    ABTIlist       rest;
    
    ConsBTlist(ConsBinTreeInt aFirst, ABTIlist aRest){
        this.first = aFirst;
        this.rest  = aRest;
    }
    
    /* produce a new list which contains all the left children
       of the tree which is the first item in this list */
    ConsBTlist insertLeft(){
    
        /* tree whose left children will be inserted into the new list */
        ConsBinTreeInt ptr = this.first;
        
        /* the new list starts with this list*/
        ConsBTlist newList = this;
        
        /* go down to the left and insert the next node */
        while (ptr.left instanceof ConsBinTreeInt){
            newList = new ConsBTlist(((ConsBinTreeInt)ptr.left), newList);
            ptr = (ConsBinTreeInt)ptr.left;
        }
        
        return newList;
    }
    
    public String toString(){
        return("*" + 
                this.first.node + "*, " +
                this.rest + "\n");
    }
}